@@ -1,49 +1,50 @@
-{-# LANGUAGE CPP               #-}
+{-# LANGUAGE NamedFieldPuns    #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards   #-}
 
 module Dhall.Import.HTTP
     ( fetchFromHttpUrl
+    , originHeadersFileExpr
     ) where
 
-import Control.Exception (Exception)
-import Control.Monad.IO.Class (MonadIO(..))
+import Control.Exception                (Exception)
+import Control.Monad.IO.Class           (MonadIO (..))
 import Control.Monad.Trans.State.Strict (StateT)
-import Data.ByteString (ByteString)
-import Data.CaseInsensitive (CI)
-import Data.Dynamic (toDyn)
-import Data.Semigroup ((<>))
-
+import Data.ByteString                  (ByteString)
+import Data.CaseInsensitive             (CI)
+import Data.Dynamic                     (toDyn)
+import Data.List.NonEmpty               (NonEmpty (..))
+import Data.Text.Encoding               (decodeUtf8)
 import Dhall.Core
-    ( Import(..)
-    , ImportHashed(..)
-    , ImportType(..)
-    , Scheme(..)
-    , URL(..)
+    ( Expr (..)
+    , Directory (..)
+    , File (..)
+    , FilePrefix (..)
+    , Import (..)
+    , ImportHashed (..)
+    , ImportMode (..)
+    , ImportType (..)
+    , Scheme (..)
+    , URL (..)
     )
-import Dhall.URL (renderURL)
-
-import qualified Control.Monad.Trans.State.Strict as State
-import qualified Data.Text                        as Text
-import qualified Data.Text.Encoding
-import qualified Dhall.Util
-
 import Dhall.Import.Types
+import Dhall.Parser                     (Src)
+import Dhall.URL                        (renderURL)
+import System.Directory                 (getXdgDirectory, XdgDirectory(XdgConfig))
+import System.FilePath                  (splitDirectories)
+
+
+import Network.HTTP.Client (HttpException (..), HttpExceptionContent (..))
 
 import qualified Control.Exception
-import qualified Data.List.NonEmpty               as NonEmpty
+import qualified Control.Monad.Trans.State.Strict as State
+import qualified Data.HashMap.Strict              as HashMap
+import qualified Data.Text                        as Text
+import qualified Data.Text.Encoding
 import qualified Data.Text.Lazy
 import qualified Data.Text.Lazy.Encoding
-
-#if MIN_VERSION_http_client(0,5,0)
-import Network.HTTP.Client
-    (HttpException(..), HttpExceptionContent(..), Manager)
-#else
-import Network.HTTP.Client (HttpException(..), Manager)
-#endif
-
-import qualified Network.HTTP.Client                     as HTTP
-import qualified Network.HTTP.Client.TLS                 as HTTP
+import qualified Dhall.Util
+import qualified Network.HTTP.Client              as HTTP
 import qualified Network.HTTP.Types
 
 mkPrettyHttpException :: String -> HttpException -> PrettyHttpException
@@ -51,7 +52,6 @@ mkPrettyHttpException url ex =
     PrettyHttpException (renderPrettyHttpException url ex) (toDyn ex)
 
 renderPrettyHttpException :: String -> HttpException -> String
-#if MIN_VERSION_http_client(0,5,0)
 renderPrettyHttpException _ (InvalidUrlException _ r) =
       "\n"
   <>  "\ESC[1;31mError\ESC[0m: Invalid URL\n"
@@ -74,6 +74,11 @@ renderPrettyHttpException url (HttpExceptionRequest _ e) =
       <>  "\ESC[1;31mError\ESC[0m: The remote host took too long to respond\n"
       <>  "\n"
       <>  "URL: " <> url <> "\n"
+    ConnectionTimeout ->
+          "\n"
+      <>  "\ESC[1;31mError\ESC[0m: Connection establishment took too long\n"
+      <>  "\n"
+      <>  "URL: " <> url <> "\n"
     StatusCodeException response body -> prefix <> suffix
       where
         prefix
@@ -150,47 +155,24 @@ renderPrettyHttpException url (HttpExceptionRequest _ e) =
 
                     prefixedText = Text.unlines prefixedLines
 
-    e' -> "\n" <> show e' <> "\n"
-#else
-renderPrettyHttpException url e = case e of
-    FailedConnectionException2 _ _ _ e' ->
-            "\n"
-        <>  "\ESC[1;31mError\ESC[0m: Wrong host:\n"
-        <>  "\n"
-        <>  "Host: " <> show e' <> "\n"
-    InvalidDestinationHost host ->
-            "\n"
-        <>  "\ESC[1;31mError\ESC[0m: Invalid host name:\n"
-        <>  "\n"
-        <>  "Host: " <> show host <> "\n"
-    ResponseTimeout ->
-            "\ESC[1;31mError\ESC[0m: The host took too long to respond\n"
-        <>  "\n"
-        <>  "URL: " <> url <> "\n"
-    e' ->   "\n"
-        <>  show e' <> "\n"
-#endif
+    e' -> "\n"
+      <>  "\ESC[1;31mError\ESC[0m: " <> show e'
+      <>  "\n"
+      <>  "URL: " <> url <> "\n"
 
 newManager :: StateT Status IO Manager
 newManager = do
-    let settings = HTTP.tlsManagerSettings
-#if MIN_VERSION_http_client(0,5,0)
-          { HTTP.managerResponseTimeout = HTTP.responseTimeoutMicro (30 * 1000 * 1000) }  -- 30 seconds
-#else
-          { HTTP.managerResponseTimeout = Just (30 * 1000 * 1000) }  -- 30 seconds
-#endif
-
     Status { _manager = oldManager, ..} <- State.get
 
     case oldManager of
         Nothing -> do
-            manager <- liftIO (HTTP.newManager settings)
+            manager <- liftIO _newManager
 
             State.put (Status { _manager = Just manager , ..})
 
             return manager
 
-        Just manager -> do
+        Just manager ->
             return manager
 
 data NotCORSCompliant = NotCORSCompliant
@@ -268,20 +250,35 @@ corsCompliant (Remote parentURL) childURL responseHeaders = liftIO $ do
                 Control.Exception.throwIO (NotCORSCompliant {..})
 corsCompliant _ _ _ = return ()
 
-type HTTPHeader = Network.HTTP.Types.Header
+addHeaders :: OriginHeaders -> Maybe [HTTPHeader] -> HTTP.Request -> HTTP.Request
+addHeaders originHeaders urlHeaders request =
+    request { HTTP.requestHeaders = (filterHeaders urlHeaders) <> perOriginHeaders }
+      where
+        origin = decodeUtf8 (HTTP.host request) <> ":" <> Text.pack (show (HTTP.port request))
+ 
+        perOriginHeaders = HashMap.lookupDefault [] origin originHeaders
+
+        filterHeaders = foldMap (filter (not . overridden))
+
+        overridden :: HTTPHeader -> Bool
+        overridden (key, _value) = any (matchesKey key) perOriginHeaders
+
+        matchesKey :: CI ByteString -> HTTPHeader -> Bool
+        matchesKey key (candidate, _value) = key == candidate
 
 fetchFromHttpUrl :: URL -> Maybe [HTTPHeader] -> StateT Status IO Text.Text
 fetchFromHttpUrl childURL mheaders = do
+    Status { _loadOriginHeaders } <- State.get
+
+    originHeaders <- _loadOriginHeaders
+
     manager <- newManager
 
     let childURLString = Text.unpack (renderURL childURL)
 
-    request <- liftIO (HTTP.parseUrlThrow childURLString)
+    baseRequest <- liftIO (HTTP.parseUrlThrow childURLString)
 
-    let requestWithHeaders =
-            case mheaders of
-              Nothing      -> request
-              Just headers -> request { HTTP.requestHeaders = headers }
+    let requestWithHeaders = addHeaders originHeaders mheaders baseRequest
 
     let io = HTTP.httpLbs requestWithHeaders manager
 
@@ -293,14 +290,26 @@ fetchFromHttpUrl childURL mheaders = do
 
     Status {..} <- State.get
 
-    let Chained parentImport = NonEmpty.head _stack
+    case _stack of
+        -- We ignore the first import in the stack since that is the same import
+        -- as the `childUrl`
+        _ :| Chained parentImport : _ -> do
+            let parentImportType = importType (importHashed parentImport)
 
-    let parentImportType = importType (importHashed parentImport)
-
-    corsCompliant parentImportType childURL (HTTP.responseHeaders response)
+            corsCompliant parentImportType childURL (HTTP.responseHeaders response)
+        _ -> do
+            return ()
 
     let bytes = HTTP.responseBody response
 
     case Data.Text.Lazy.Encoding.decodeUtf8' bytes of
         Left  err  -> liftIO (Control.Exception.throwIO err)
         Right text -> return (Data.Text.Lazy.toStrict text)
+
+originHeadersFileExpr :: IO (Expr Src Import)
+originHeadersFileExpr = do
+    directoryStr <- getXdgDirectory XdgConfig "dhall"
+    let components = map Text.pack (splitDirectories directoryStr)
+    let directory = Directory (reverse components)
+    let file = (File directory "headers.dhall")
+    return (Embed (Import (ImportHashed Nothing (Local Absolute file)) Code))
\ No newline at end of file
