@@ -7,14 +7,13 @@
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE MultiParamTypeClasses      #-}
-{-# LANGUAGE MultiWayIf                 #-}
+{-# LANGUAGE NamedFieldPuns             #-}
 {-# LANGUAGE OverloadedStrings          #-}
 {-# LANGUAGE PolyKinds                  #-}
 {-# LANGUAGE RankNTypes                 #-}
 {-# LANGUAGE RecordWildCards            #-}
 {-# LANGUAGE ScopedTypeVariables        #-}
 {-# LANGUAGE StandaloneDeriving         #-}
-{-# LANGUAGE TupleSections              #-}
 {-# LANGUAGE TypeApplications           #-}
 {-# LANGUAGE TypeFamilies               #-}
 {-# LANGUAGE TypeOperators              #-}
@@ -56,6 +55,7 @@ module Dhall.Marshal.Decode
     , string
     , lazyText
     , strictText
+    , shortText
       -- ** Time
     , timeOfDay
     , day
@@ -63,6 +63,7 @@ module Dhall.Marshal.Decode
     , localTime
     , zonedTime
     , utcTime
+    , dayOfWeek
       -- ** Containers
     , maybe
     , pair
@@ -146,7 +147,6 @@ import Data.Functor.Contravariant
     , Predicate (..)
     )
 import Data.Hashable                    (Hashable)
-import Data.Int                         (Int16, Int32, Int64, Int8)
 import Data.List.NonEmpty               (NonEmpty (..))
 import Data.Typeable                    (Proxy (..), Typeable)
 import Dhall.Parser                     (Src (..))
@@ -154,6 +154,7 @@ import Dhall.Syntax
     ( Chunks (..)
     , DhallDouble (..)
     , Expr (..)
+    , FieldSelection (..)
     , FunctionBinding (..)
     , Var (..)
     )
@@ -176,6 +177,7 @@ import qualified Data.Sequence
 import qualified Data.Set
 import qualified Data.Text
 import qualified Data.Text.Lazy
+import qualified Data.Text.Short
 import qualified Data.Time            as Time
 import qualified Data.Vector
 import qualified Dhall.Core           as Core
@@ -303,6 +305,9 @@ instance FromDhall Double where
 instance {-# OVERLAPS #-} FromDhall [Char] where
     autoWith _ = string
 
+instance FromDhall Data.Text.Short.ShortText where
+    autoWith _ = shortText
+
 instance FromDhall Data.Text.Lazy.Text where
     autoWith _ = lazyText
 
@@ -339,6 +344,9 @@ instance FromDhall Time.ZonedTime where
 instance FromDhall Time.UTCTime where
     autoWith _ = utcTime
 
+instance FromDhall Time.DayOfWeek where
+    autoWith _ = dayOfWeek
+
 {-| Note that this instance will throw errors in the presence of duplicates in
     the list. To ignore duplicates, use `setIgnoringDuplicates`.
 -}
@@ -909,6 +917,14 @@ double = Decoder {..}
 
     expected = pure Double
 
+{-| Decode `Data.Text.Short.ShortText`.
+
+>>> input shortText "\"Test\""
+"Test"
+-}
+shortText :: Decoder Data.Text.Short.ShortText
+shortText = fmap Data.Text.Short.fromText strictText
+
 {-| Decode lazy `Data.Text.Text`.
 
 >>> input lazyText "\"Test\""
@@ -1002,6 +1018,42 @@ zonedTime = record $
 utcTime :: Decoder Time.UTCTime
 utcTime = Time.zonedTimeToUTC <$> zonedTime
 
+{-| Decode `Time.DayOfWeek`
+
+>>> input dayOfWeek "< Sunday | Monday | Tuesday | Wednesday | Thursday | Friday | Saturday >.Monday"
+Monday
+-}
+dayOfWeek :: Decoder Time.DayOfWeek
+dayOfWeek = Decoder{..}
+  where
+    extract expr@(Field _ FieldSelection{ fieldSelectionLabel }) =
+        case fieldSelectionLabel of
+            "Sunday"    -> pure Time.Sunday
+            "Monday"    -> pure Time.Monday
+            "Tuesday"   -> pure Time.Tuesday
+            "Wednesday" -> pure Time.Wednesday
+            "Thursday"  -> pure Time.Thursday
+            "Friday"    -> pure Time.Friday
+            "Saturday"  -> pure Time.Saturday
+            _           -> typeError expected expr
+    extract expr =
+        typeError expected expr
+
+    expected =
+        pure
+            (Union
+                (Dhall.Map.fromList
+                    [ ("Sunday", Nothing)
+                    , ("Monday", Nothing)
+                    , ("Tuesday", Nothing)
+                    , ("Wednesday", Nothing)
+                    , ("Thursday", Nothing)
+                    , ("Friday", Nothing)
+                    , ("Saturday", Nothing)
+                    ]
+                )
+            )
+
 {-| Decode a `Maybe`.
 
 >>> input (maybe natural) "Some 1"
