@@ -234,26 +234,29 @@ temporalLiteral =
     <|> try partialTime
     <|> try timeNumOffset
 
+-- | Parse a \"shebang\" line (i.e. an initial line beginning with @#!@)
+shebang :: Parser ()
+shebang = do
+    _ <- text "#!"
+
+    let predicate c = ('\x20' <= c && c <= '\x10FFFF') || c == '\t'
+
+    _ <- Dhall.Parser.Combinators.takeWhile predicate
+
+    _ <- endOfLine
+
+    return ()
+
 -- | Given a parser for imports,
 parsers :: forall a. Parser a -> Parsers a
 parsers embedded = Parsers{..}
   where
     completeExpression_ =
-            many shebang
-        *>  whitespace
+            whitespace
         *>  expression
         <*  whitespace
         <*  optional lineCommentPrefix
 
-    shebang = do
-        _ <- text "#!"
-
-        let predicate c = ('\x20' <= c && c <= '\x10FFFF') || c == '\t'
-
-        _ <- Dhall.Parser.Combinators.takeWhile predicate
-
-        endOfLine
-
     letBinding = do
         src0 <- try (_let *> src nonemptyWhitespace)
 
@@ -385,7 +388,11 @@ parsers embedded = Parsers{..}
                     bs <- some (do
                         try (nonemptyWhitespace *> _with *> nonemptyWhitespace)
 
-                        keys <- Combinators.NonEmpty.sepBy1 anyLabel (try (whitespace *> _dot) *> whitespace)
+                        let withComponent =
+                                    fmap WithLabel anyLabelOrSome
+                                <|> fmap (\_ -> WithQuestion) (text "?")
+
+                        keys <- Combinators.NonEmpty.sepBy1 withComponent (try (whitespace *> _dot) *> whitespace)
 
                         whitespace
 
@@ -517,10 +524,15 @@ parsers embedded = Parsers{..}
 
                     return (\a -> ToMap a Nothing, Just "argument to ❰toMap❱")
 
-            let alternative3 =
+            let alternative3 = do
+                    try (_showConstructor *> nonemptyWhitespace)
+
+                    return (\a -> ShowConstructor a, Just "argument to ❰showConstructor❱")
+
+            let alternative4 =
                     return (id, Nothing)
 
-            (f, maybeMessage) <- alternative0 <|> alternative1 <|> alternative2 <|> alternative3
+            (f, maybeMessage) <- alternative0 <|> alternative1 <|> alternative2 <|> alternative3 <|> alternative4
 
             let adapt parser =
                     case maybeMessage of
