@@ -8,6 +8,7 @@ module Dhall.Pretty
     , prettyExpr
 
     , CharacterSet(..)
+    , detectCharacterSet
     , prettyCharacterSet
 
     , Dhall.Pretty.Internal.layout
@@ -15,6 +16,8 @@ module Dhall.Pretty
 
     , escapeEnvironmentVariable
     , escapeLabel
+
+    , temporalToText
     ) where
 
 import Dhall.Pretty.Internal
