@@ -1,4 +1,3 @@
-{-# LANGUAGE CPP               #-}
 {-# LANGUAGE DeriveTraversable #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE PatternGuards     #-}
@@ -7,51 +6,55 @@
 
 -- | Template Haskell utilities
 module Dhall.TH
-    ( -- * Template Haskell
+    ( -- * Embedding Dhall in Haskell
       staticDhallExpression
+    , dhall
+      -- * Generating Haskell from Dhall expressions
     , makeHaskellTypeFromUnion
     , makeHaskellTypes
+    , makeHaskellTypesWith
     , HaskellType(..)
+    , GenerateOptions(..)
+    , defaultGenerateOptions
     ) where
 
-import Data.Monoid ((<>))
-import Data.Text (Text)
-import Data.Text.Prettyprint.Doc (Pretty)
-import Dhall.Syntax (Expr(..))
-import Dhall (FromDhall, ToDhall)
-import GHC.Generics (Generic)
-import Language.Haskell.TH.Quote (dataToExpQ) -- 7.10 compatibility.
+import Data.Text                 (Text)
+import Dhall                     (FromDhall, ToDhall)
+import Dhall.Syntax              (Expr (..))
+import GHC.Generics              (Generic)
+import Language.Haskell.TH.Quote (QuasiQuoter (..), dataToExpQ)
+import Prettyprinter             (Pretty)
 
 import Language.Haskell.TH.Syntax
-    ( Con(..)
-    , Dec(..)
-    , Exp(..)
+    ( Bang (..)
+    , Body (..)
+    , Con (..)
+    , Dec (..)
+    , Exp (..)
+    , Match (..)
+    , Pat (..)
     , Q
-    , Type(..)
-    , Bang(..)
-    , SourceStrictness(..)
-    , SourceUnpackedness(..)
-#if MIN_VERSION_template_haskell(2,12,0)
-    , DerivClause(..)
-    , DerivStrategy(..)
-#else
-    , Pred
-#endif
+    , SourceStrictness (..)
+    , SourceUnpackedness (..)
+    , Type (..)
     )
 
-import qualified Data.List                               as List
-import qualified Data.Text                               as Text
-import qualified Data.Text.Prettyprint.Doc.Render.String as Pretty
-import qualified Data.Typeable                           as Typeable
+import Language.Haskell.TH.Syntax (DerivClause (..), DerivStrategy (..))
+
+import qualified Data.List                   as List
+import qualified Data.Set                    as Set
+import qualified Data.Text                   as Text
+import qualified Data.Typeable               as Typeable
 import qualified Dhall
-import qualified Dhall.Core                              as Core
+import qualified Dhall.Core                  as Core
 import qualified Dhall.Map
 import qualified Dhall.Pretty
 import qualified Dhall.Util
 import qualified GHC.IO.Encoding
+import qualified Language.Haskell.TH.Syntax  as Syntax
 import qualified Numeric.Natural
+import qualified Prettyprinter.Render.String as Pretty
 import qualified System.IO
-import qualified Language.Haskell.TH.Syntax              as Syntax
 
 {-| This fully resolves, type checks, and normalizes the expression, so the
     resulting AST is self-contained.
@@ -64,8 +67,8 @@ import qualified Language.Haskell.TH.Syntax              as Syntax
 
     > < This : Natural | Other : ../Other/Type.dhall >
 
-    ... rather than duplicating the AST manually in a Haskell `Type`, you can
-    do:
+    ... rather than duplicating the AST manually in a Haskell `Dhall.Type`, you
+    can do:
 
     > Dhall.Type
     > (\case
@@ -83,12 +86,29 @@ staticDhallExpression text = do
 
     expression <- Syntax.runIO (Dhall.inputExpr text)
 
-    dataToExpQ (\a -> liftText <$> Typeable.cast a) expression
+    dataToExpQ (fmap liftText . Typeable.cast) expression
   where
     -- A workaround for a problem in TemplateHaskell (see
     -- https://stackoverflow.com/questions/38143464/cant-find-inerface-file-declaration-for-variable)
     liftText = fmap (AppE (VarE 'Text.pack)) . Syntax.lift . Text.unpack
 
+{-| A quasi-quoter for Dhall expressions.
+
+    This quoter is build on top of 'staticDhallExpression'. Therefore consult the
+    documentation of that function for further information.
+
+    This quoter is meant to be used in expression context only; Other contexts
+    like pattern contexts or declaration contexts are not supported and will
+    result in an error.
+-}
+dhall :: QuasiQuoter
+dhall = QuasiQuoter
+    { quoteExp = staticDhallExpression . Text.pack
+    , quotePat = const $ error "dhall quasi-quoter: Quoting patterns is not supported!"
+    , quoteType = const $ error "dhall quasi-quoter: Quoting types is not supported!"
+    , quoteDec = const $ error "dhall quasi-quoter: Quoting declarations is not supported!"
+    }
+
 {-| Convert a Dhall type to a Haskell type that does not require any new
     data declarations beyond the data declarations supplied as the first
     argument
@@ -106,19 +126,19 @@ toNestedHaskellType
 toNestedHaskellType haskellTypes = loop
   where
     loop dhallType = case dhallType of
-        Bool -> do
+        Bool ->
             return (ConT ''Bool)
 
-        Double -> do
+        Double ->
             return (ConT ''Double)
 
-        Integer -> do
+        Integer ->
             return (ConT ''Integer)
 
-        Natural -> do
+        Natural ->
             return (ConT ''Numeric.Natural.Natural)
 
-        Text -> do
+        Text ->
             return (ConT ''Text)
 
         App List dhallElementType -> do
@@ -134,7 +154,7 @@ toNestedHaskellType haskellTypes = loop
         _   | Just haskellType <- List.find predicate haskellTypes -> do
                 let name = Syntax.mkName (Text.unpack (typeName haskellType))
 
-                return (ConT name) 
+                return (ConT name)
             | otherwise -> do
             let document =
                     mconcat
@@ -167,31 +187,53 @@ toNestedHaskellType haskellTypes = loop
             predicate haskellType =
                 Core.judgmentallyEqual (code haskellType) dhallType
 
-#if MIN_VERSION_template_haskell(2,12,0)
-derivingClauses :: [DerivClause]
-derivingClauses =
-    [ DerivClause (Just StockStrategy) [ ConT ''Generic ]
-    , DerivClause (Just AnyclassStrategy) [ ConT ''FromDhall, ConT ''ToDhall ]
-    ]
-#else
-derivingClauses :: [Pred]
-derivingClauses = [ ConT ''Generic, ConT ''FromDhall, ConT ''ToDhall ]
-#endif
+-- | A deriving clause for `Generic`.
+derivingGenericClause :: DerivClause
+derivingGenericClause = DerivClause (Just StockStrategy) [ ConT ''Generic ]
+
+-- | Generates a `FromDhall` instances.
+fromDhallInstance
+    :: Syntax.Name -- ^ The name of the type the instances is for
+    -> Q Exp       -- ^ A TH splice generating some `Dhall.InterpretOptions`
+    -> Q [Dec]
+fromDhallInstance n interpretOptions = [d|
+    instance FromDhall $(pure $ ConT n) where
+        autoWith = Dhall.genericAutoWithInputNormalizer $(interpretOptions)
+    |]
+
+-- | Generates a `ToDhall` instances.
+toDhallInstance
+    :: Syntax.Name -- ^ The name of the type the instances is for
+    -> Q Exp       -- ^ A TH splice generating some `Dhall.InterpretOptions`
+    -> Q [Dec]
+toDhallInstance n interpretOptions = [d|
+    instance ToDhall $(pure $ ConT n) where
+        injectWith = Dhall.genericToDhallWithInputNormalizer $(interpretOptions)
+    |]
 
 -- | Convert a Dhall type to the corresponding Haskell datatype declaration
 toDeclaration
     :: (Eq a, Pretty a)
-    => [HaskellType (Expr s a)]
+    => GenerateOptions
+    -> [HaskellType (Expr s a)]
     -> HaskellType (Expr s a)
-    -> Q Dec
-toDeclaration haskellTypes MultipleConstructors{..} = do
+    -> Q [Dec]
+toDeclaration generateOptions@GenerateOptions{..} haskellTypes typ@MultipleConstructors{..} =
     case code of
         Union kts -> do
             let name = Syntax.mkName (Text.unpack typeName)
 
-            constructors <- traverse (toConstructor haskellTypes) (Dhall.Map.toList kts )
+            let derivingClauses =
+                    [ derivingGenericClause | generateFromDhallInstance || generateToDhallInstance ]
 
-            return (DataD [] name [] Nothing constructors derivingClauses)
+            constructors <- traverse (toConstructor generateOptions haskellTypes typeName) (Dhall.Map.toList kts)
+
+            let interpretOptions = generateToInterpretOptions generateOptions typ
+
+            fmap concat . sequence $
+                [pure [DataD [] name [] Nothing constructors derivingClauses]] <>
+                [ fromDhallInstance name interpretOptions | generateFromDhallInstance ] <>
+                [ toDhallInstance name interpretOptions | generateToDhallInstance ]
 
         _ -> do
             let document =
@@ -235,33 +277,54 @@ toDeclaration haskellTypes MultipleConstructors{..} = do
             let message = Pretty.renderString (Dhall.Pretty.layout document)
 
             fail message
-toDeclaration haskellTypes SingleConstructor{..} = do
+toDeclaration generateOptions@GenerateOptions{..} haskellTypes typ@SingleConstructor{..} = do
     let name = Syntax.mkName (Text.unpack typeName)
 
-    constructor <- toConstructor haskellTypes (constructorName, Just code)
+    let derivingClauses =
+            [ derivingGenericClause | generateFromDhallInstance || generateToDhallInstance ]
 
-    return (DataD [] name [] Nothing [constructor] derivingClauses)
+    let interpretOptions = generateToInterpretOptions generateOptions typ
+
+    constructor <- toConstructor generateOptions haskellTypes typeName (constructorName, Just code)
+
+    fmap concat . sequence $
+        [pure [DataD [] name [] Nothing [constructor] derivingClauses]] <>
+        [ fromDhallInstance name interpretOptions | generateFromDhallInstance ] <>
+        [ toDhallInstance name interpretOptions | generateToDhallInstance ]
 
 -- | Convert a Dhall type to the corresponding Haskell constructor
 toConstructor
     :: (Eq a, Pretty a)
-    => [HaskellType (Expr s a)]
+    => GenerateOptions
+    -> [HaskellType (Expr s a)]
+    -> Text
+    -- ^ typeName
     -> (Text, Maybe (Expr s a))
     -- ^ @(constructorName, fieldType)@
     -> Q Con
-toConstructor haskellTypes (constructorName, maybeAlternativeType) = do
-    let name = Syntax.mkName (Text.unpack constructorName)
+toConstructor GenerateOptions{..} haskellTypes outerTypeName (constructorName, maybeAlternativeType) = do
+    let name = Syntax.mkName (Text.unpack $ constructorModifier constructorName)
 
     let bang = Bang NoSourceUnpackedness NoSourceStrictness
 
     case maybeAlternativeType of
+        Just dhallType
+            | let predicate haskellType =
+                    Core.judgmentallyEqual (code haskellType) dhallType
+                    && typeName haskellType /= outerTypeName
+            , Just haskellType <- List.find predicate haskellTypes -> do
+                let innerName =
+                        Syntax.mkName (Text.unpack (typeName haskellType))
+
+                return (NormalC name [ (bang, ConT innerName) ])
+
         Just (Record kts) -> do
             let process (key, dhallFieldType) = do
                     haskellFieldType <- toNestedHaskellType haskellTypes dhallFieldType
 
-                    return (Syntax.mkName (Text.unpack key), bang, haskellFieldType)
+                    return (Syntax.mkName (Text.unpack $ fieldModifier key), bang, haskellFieldType)
 
-            varBangTypes <- traverse process (Dhall.Map.toList kts)
+            varBangTypes <- traverse process (Dhall.Map.toList $ Core.recordFieldValue <$> kts)
 
             return (RecC name varBangTypes)
 
@@ -270,7 +333,7 @@ toConstructor haskellTypes (constructorName, maybeAlternativeType) = do
 
             return (NormalC name [ (bang, haskellAlternativeType) ])
 
-        Nothing -> do
+        Nothing ->
             return (NormalC name [])
 
 -- | Generate a Haskell datatype declaration from a Dhall union type where
@@ -297,17 +360,18 @@ makeHaskellTypeFromUnion
 makeHaskellTypeFromUnion typeName code =
     makeHaskellTypes [ MultipleConstructors{..} ]
 
--- | Used by `makeHaskellTypes` to specify how to generate Haskell types
+-- | Used by `makeHaskellTypes` and `makeHaskellTypesWith` to specify how to
+-- generate Haskell types.
 data HaskellType code
     -- | Generate a Haskell type with more than one constructor from a Dhall
-    -- union type
+    -- union type.
     = MultipleConstructors
         { typeName :: Text
         -- ^ Name of the generated Haskell type
         , code :: code
         -- ^ Dhall code that evaluates to a union type
         }
-    -- | Generate a Haskell type with one constructor from any Dhall type
+    -- | Generate a Haskell type with one constructor from any Dhall type.
     --
     -- To generate a constructor with multiple named fields, supply a Dhall
     -- record type.  This does not support more than one anonymous field.
@@ -321,8 +385,82 @@ data HaskellType code
         }
     deriving (Functor, Foldable, Traversable)
 
+-- | This data type holds various options that let you control several aspects
+-- how Haskell code is generated. In particular you can
+--
+--   * disable the generation of `FromDhall`/`ToDhall` instances.
+--   * modify how a Dhall union field translates to a Haskell data constructor.
+data GenerateOptions = GenerateOptions
+    { constructorModifier :: Text -> Text
+    -- ^ How to map a Dhall union field name to a Haskell constructor.
+    -- Note: The `constructorName` of `SingleConstructor` will be passed to this function, too.
+    , fieldModifier :: Text -> Text
+    -- ^ How to map a Dhall record field names to a Haskell record field names.
+    , generateFromDhallInstance :: Bool
+    -- ^ Generate a `FromDhall` instance for the Haskell type
+    , generateToDhallInstance :: Bool
+    -- ^ Generate a `ToDhall` instance for the Haskell type
+    }
+
+-- | A default set of options used by `makeHaskellTypes`. That means:
+--
+--     * Constructors and fields are passed unmodified.
+--     * Both `FromDhall` and `ToDhall` instances are generated.
+defaultGenerateOptions :: GenerateOptions
+defaultGenerateOptions = GenerateOptions
+    { constructorModifier = id
+    , fieldModifier = id
+    , generateFromDhallInstance = True
+    , generateToDhallInstance = True
+    }
+
+-- | This function generates `Dhall.InterpretOptions` that can be used for the
+--   marshalling of the Haskell type generated according to the `GenerateOptions`.
+--   I.e. those `Dhall.InterpretOptions` reflect the mapping done by
+--   `constructorModifier` and `fieldModifier` on the value level.
+generateToInterpretOptions :: GenerateOptions -> HaskellType (Expr s a) -> Q Exp
+generateToInterpretOptions GenerateOptions{..} haskellType = [| Dhall.InterpretOptions
+    { Dhall.fieldModifier = \ $(pure nameP) ->
+        $(toCases fieldModifier $ fields haskellType)
+    , Dhall.constructorModifier = \ $(pure nameP) ->
+        $(toCases constructorModifier $ constructors haskellType)
+    , Dhall.singletonConstructors = Dhall.singletonConstructors Dhall.defaultInterpretOptions
+    }|]
+    where
+        constructors :: HaskellType (Expr s a) -> [Text]
+        constructors SingleConstructor{..} = [constructorName]
+        constructors MultipleConstructors{..} | Union kts <- code = Dhall.Map.keys kts
+        constructors _ = []
+
+        fields :: HaskellType (Expr s a) -> [Text]
+        fields SingleConstructor{..} | Record kts <- code = Dhall.Map.keys kts
+        fields MultipleConstructors{..} | Union kts <- code = Set.toList $ mconcat
+            [ Dhall.Map.keysSet kts'
+            | (_, Just (Record kts')) <- Dhall.Map.toList kts
+            ]
+        fields _ = []
+
+        toCases :: (Text -> Text) -> [Text] -> Q Exp
+        toCases f xs = do
+            err <- [| Core.internalError $ "Unmatched " <> Text.pack (show $(pure nameE)) |]
+            pure $ CaseE nameE $ map mkMatch xs <> [Match WildP (NormalB err) []]
+            where
+                mkMatch n = Match (textToPat $ f n) (NormalB $ textToExp n) []
+
+        nameE :: Exp
+        nameE = Syntax.VarE $ Syntax.mkName "n"
+
+        nameP :: Pat
+        nameP = Syntax.VarP $ Syntax.mkName "n"
+
+        textToExp :: Text -> Exp
+        textToExp = Syntax.LitE . Syntax.StringL . Text.unpack
+
+        textToPat :: Text -> Pat
+        textToPat = Syntax.LitP . Syntax.StringL . Text.unpack
+
 -- | Generate a Haskell datatype declaration with one constructor from a Dhall
--- type
+-- type.
 --
 -- This comes in handy if you need to keep Dhall types and Haskell types in
 -- sync.  You make the Dhall types the source of truth and use Template Haskell
@@ -374,7 +512,7 @@ data HaskellType code
 --
 -- By default, the generated types only derive `GHC.Generics.Generic`,
 -- `Dhall.FromDhall`, and `Dhall.ToDhall`.  To add any desired instances (such
--- as `Eq`\/`Ord`\/`Show`), you can use the `StandaloneDeriving` language
+-- as `Eq`\/`Ord`\/`Show`), you can use the @StandaloneDeriving@ language
 -- extension, like this:
 --
 -- > {-# LANGUAGE DeriveAnyClass     #-}
@@ -397,9 +535,18 @@ data HaskellType code
 -- > deriving instance Ord  Employee
 -- > deriving instance Show Employee
 makeHaskellTypes :: [HaskellType Text] -> Q [Dec]
-makeHaskellTypes haskellTypes = do
+makeHaskellTypes = makeHaskellTypesWith defaultGenerateOptions
+
+-- | Like `makeHaskellTypes`, but with the ability to customize the generated
+-- Haskell code by passing `GenerateOptions`.
+--
+-- For instance, `makeHaskellTypes` is implemented using this function:
+--
+-- > makeHaskellTypes = makeHaskellTypesWith defaultGenerateOptions
+makeHaskellTypesWith :: GenerateOptions -> [HaskellType Text] -> Q [Dec]
+makeHaskellTypesWith generateOptions haskellTypes = do
     Syntax.runIO (GHC.IO.Encoding.setLocaleEncoding System.IO.utf8)
 
     haskellTypes' <- traverse (traverse (Syntax.runIO . Dhall.inputExpr)) haskellTypes
 
-    traverse (toDeclaration haskellTypes') haskellTypes'
+    concat <$> traverse (toDeclaration generateOptions haskellTypes') haskellTypes'
