@@ -2,13 +2,12 @@
 
 module Dhall.Test.Normalization where
 
-import Data.Monoid ((<>))
-import Data.Text (Text)
-import Data.Void (Void)
-import Dhall.Core (Expr(..), Var(..), throws)
-import Prelude hiding (FilePath)
+import Data.Text  (Text)
+import Data.Void  (Void)
+import Dhall.Core (Expr (..), Var (..), throws)
+import Prelude    hiding (FilePath)
 import Test.Tasty (TestTree)
-import Turtle (FilePath, (</>))
+import Turtle     (FilePath, (</>))
 
 import qualified Data.Text        as Text
 import qualified Data.Text.IO     as Text.IO
@@ -68,7 +67,7 @@ simpleCustomization = Tasty.HUnit.testCase "simpleCustomization" $ do
     let tyCtx =
             Context.insert
                 "min"
-                (Pi "_" Natural (Pi "_" Natural Natural))
+                (Pi mempty "_" Natural (Pi mempty "_" Natural Natural))
                 Context.empty
 
         valCtx e =
