@@ -5,30 +5,31 @@
 
 module Dhall.Test.Regression where
 
+import Data.Either.Validation (Validation (..))
+import Data.Void              (Void)
+import Dhall.Import           (Imported, MissingImports (..))
+import Dhall.Parser           (SourcedException (..), Src)
+import Dhall.TypeCheck        (TypeError)
+import Test.Tasty             (TestTree)
+import Test.Tasty.HUnit       ((@?=))
+
 import qualified Control.Exception
 import qualified Data.Text.IO
 import qualified Data.Text.Lazy.IO
-import qualified Data.Text.Prettyprint.Doc
-import qualified Data.Text.Prettyprint.Doc.Render.Text
 import qualified Dhall
 import qualified Dhall.Context
 import qualified Dhall.Core
 import qualified Dhall.Map
 import qualified Dhall.Parser
 import qualified Dhall.Pretty
-import qualified Dhall.Test.Util as Util
+import qualified Dhall.Test.Util           as Util
 import qualified Dhall.TypeCheck
+import qualified Prettyprinter
+import qualified Prettyprinter.Render.Text
 import qualified System.Timeout
 import qualified Test.Tasty
 import qualified Test.Tasty.HUnit
 
-import Data.Void (Void)
-import Dhall.Import (Imported, MissingImports(..))
-import Dhall.Parser (Src, SourcedException(..))
-import Dhall.TypeCheck (TypeError)
-import Test.Tasty (TestTree)
-import Test.Tasty.HUnit ((@?=))
-
 tests :: TestTree
 tests =
     Test.Tasty.testGroup "regression tests"
@@ -45,6 +46,8 @@ tests =
         , issue1584
         , issue1646
         , issue1732
+        , issue1884
+        , issue2088
         , parsing0
         , typeChecking0
         , typeChecking1
@@ -60,27 +63,37 @@ unnamedFields :: TestTree
 unnamedFields = Test.Tasty.HUnit.testCase "Unnamed Fields" (do
     let ty = Dhall.auto :: Dhall.Decoder Foo
     Test.Tasty.HUnit.assertEqual "Good type" (Dhall.expected ty)
-        (Dhall.Core.Union
+        (Success (Dhall.Core.Union
             (Dhall.Map.fromList
-                [   ("Foo",Just (Dhall.Core.Record (Dhall.Map.fromList [
-                        ("_1",Dhall.Core.Integer),("_2",Dhall.Core.Bool)])))
-                ,   ("Bar",Just (Dhall.Core.Record (Dhall.Map.fromList [
-                        ("_1",Dhall.Core.Bool),("_2",Dhall.Core.Bool),("_3",Dhall.Core.Bool)])))
-                ,   ("Baz",Just (Dhall.Core.Record (Dhall.Map.fromList [
-                        ("_1",Dhall.Core.Integer),("_2",Dhall.Core.Integer)])))
+                [   ("Foo",Just (Dhall.Core.Record (Dhall.Map.fromList
+                        [ ("_1", Dhall.Core.makeRecordField Dhall.Core.Integer)
+                        , ("_2", Dhall.Core.makeRecordField Dhall.Core.Bool)
+                        ])))
+                ,   ("Bar",Just (Dhall.Core.Record (Dhall.Map.fromList
+                        [ ("_1", Dhall.Core.makeRecordField Dhall.Core.Bool)
+                        , ("_2", Dhall.Core.makeRecordField Dhall.Core.Bool)
+                        , ("_3", Dhall.Core.makeRecordField Dhall.Core.Bool)
+                        ])))
+                ,   ("Baz",Just (Dhall.Core.Record (Dhall.Map.fromList
+                        [ ("_1", Dhall.Core.makeRecordField Dhall.Core.Integer)
+                        , ("_2", Dhall.Core.makeRecordField Dhall.Core.Integer)
+                        ])))
                 ]
             )
-        )
+        ))
 
     let inj = Dhall.inject :: Dhall.Encoder Foo
-    Test.Tasty.HUnit.assertEqual "Good ToDhall" (Dhall.declared inj) (Dhall.expected ty)
+    Test.Tasty.HUnit.assertEqual "Good ToDhall" (Success $ Dhall.declared inj) (Dhall.expected ty)
 
     let tu_ty = Dhall.auto :: Dhall.Decoder (Integer, Bool)
-    Test.Tasty.HUnit.assertEqual "Auto Tuple" (Dhall.expected tu_ty) (Dhall.Core.Record (
-            Dhall.Map.fromList [ ("_1",Dhall.Core.Integer),("_2",Dhall.Core.Bool) ]))
+    Test.Tasty.HUnit.assertEqual "Auto Tuple" (Dhall.expected tu_ty) (Success $ Dhall.Core.Record (
+            Dhall.Map.fromList
+                [ ("_1", Dhall.Core.makeRecordField Dhall.Core.Integer)
+                , ("_2", Dhall.Core.makeRecordField Dhall.Core.Bool)
+                ]))
 
     let tu_in = Dhall.inject :: Dhall.Encoder (Integer, Bool)
-    Test.Tasty.HUnit.assertEqual "Inj. Tuple" (Dhall.declared tu_in) (Dhall.expected tu_ty)
+    Test.Tasty.HUnit.assertEqual "Inj. Tuple" (Success $ Dhall.declared tu_in) (Dhall.expected tu_ty)
 
     return () )
 
@@ -103,7 +116,7 @@ issue151 = Test.Tasty.HUnit.testCase "Issue #151" (do
                     case Control.Exception.fromException e :: Maybe (Imported (TypeError Src Void)) of
                         Just _ -> return True
                         Nothing -> return False
-                handler _ = do
+                handler _ =
                     return True
 
             let typeCheck = do
@@ -149,9 +162,9 @@ issue216 = Test.Tasty.HUnit.testCase "Issue #216" (do
     -- Verify that pretty-printing preserves string interpolation
     source <- Data.Text.IO.readFile "./tests/regression/issue216b.dhall"
     e <- Util.code source
-    let doc       = Data.Text.Prettyprint.Doc.pretty e
+    let doc       = Prettyprinter.pretty e
     let docStream = Dhall.Pretty.layout doc
-    let text0 = Data.Text.Prettyprint.Doc.Render.Text.renderLazy docStream
+    let text0 = Prettyprinter.Render.Text.renderLazy docStream
 
     text1 <- Data.Text.Lazy.IO.readFile "./tests/regression/issue216b.dhall"
 
@@ -204,8 +217,22 @@ issue1732 = Test.Tasty.HUnit.testCase "Issue #1732" (do
     _ <- Util.code "./tests/regression/issue1732.dhall"
     return () )
 
+issue1884 :: TestTree
+issue1884 = Test.Tasty.HUnit.testCase "Issue #1884" (do
+    -- This test ensures that the parser allows a parenthesized application
+    -- expression as the first argument to a with expression
+    _ <- Util.code "./tests/regression/issue1884.dhall"
+    return () )
+
+issue2088 :: TestTree
+issue2088 = Test.Tasty.HUnit.testCase "Issue #2088" (do
+    -- This test ensures that the parser for projection by labels doesn't
+    -- accidentally swallow trailing commas OUTSIDE of the projection
+    _ <- Util.code "./tests/regression/issue2088.dhall"
+    return () )
+
 parsing0 :: TestTree
-parsing0 = Test.Tasty.HUnit.testCase "Parsing regression #0" (do
+parsing0 = Test.Tasty.HUnit.testCase "Parsing regression #0" (
     -- Verify that parsing should not fail
     --
     -- In 267093f8cddf1c2f909f2d997c31fd0a7cb2440a I broke the parser when left
